@@ -1,7 +1,5 @@
-package com.ibm.oti.shared;
-
 /*******************************************************************************
- * Copyright (c) 2010, 2019 IBM Corp. and others
+ * Copyright (c) 2010, 2020 IBM Corp. and others
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -21,6 +19,7 @@ package com.ibm.oti.shared;
  *
  * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
  *******************************************************************************/
+package com.ibm.oti.shared;
 
 /**
  * SharedClassCacheInfo stores information about a shared class cache and
@@ -84,6 +83,7 @@ public class SharedClassCacheInfo {
 	private long freeBytes;
 	private int cacheType;
 	private long softMaxBytes;
+	private int layer;
 	
 	/**
 	 * Gets the cache name for the shared class cache.
@@ -235,6 +235,15 @@ public class SharedClassCacheInfo {
 		}
 		return ret;
 	}
+	
+	/**
+	 * Get the layer number for the shared class cache. 
+	 *
+	 * @return		The shared cache layer number or -1 if the shared cache does not have a layer number 
+	 */		
+	public int getCacheLayer() {
+		return layer;
+	}
 
 	/**
 	 * Constructor to create an object of SharedClassCacheInfo using the parameter specified.
@@ -252,6 +261,7 @@ public class SharedClassCacheInfo {
 	 * @param		freeBytes		Free bytes in cache
 	 * @param		cacheType		Cache type
 	 * @param		softMaxBytes	Soft limit for the available space in bytes
+	 * @param		layer			The layer number
 	 */		
 	SharedClassCacheInfo(String name,
 						boolean isCompatible, 
@@ -265,7 +275,8 @@ public class SharedClassCacheInfo {
 						long cacheSize, 
 						long freeBytes,
 						int cacheType,
-						long softMaxBytes)
+						long softMaxBytes,
+						int layer)
 	{
 		this.name = name;
 		this.isCompatible = isCompatible;
@@ -280,5 +291,6 @@ public class SharedClassCacheInfo {
 		this.freeBytes = freeBytes;
 		this.cacheType = cacheType;
 		this.softMaxBytes = softMaxBytes;
+		this.layer = layer;
 	}
 }
