@@ -1,8 +1,5 @@
-
-package java.lang;
-
 /*******************************************************************************
- * Copyright (c) 1998, 2019 IBM Corp. and others
+ * Copyright (c) 1998, 2020 IBM Corp. and others
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -22,7 +19,7 @@ package java.lang;
  *
  * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
  *******************************************************************************/
- 
+package java.lang;
 
 /**
  * ThreadGroups are containers of Threads and ThreadGroups, therefore providing
@@ -36,7 +33,6 @@ package java.lang;
  * @see			Thread
  * @see			SecurityManager
  */
-
 public class ThreadGroup implements Thread.UncaughtExceptionHandler {
 
 	private String name;					// Name of this ThreadGroup		
@@ -47,22 +43,22 @@ public class ThreadGroup implements Thread.UncaughtExceptionHandler {
 	int numGroups;					// The number of children groups
 	private ThreadGroup[] childrenGroups = new ThreadGroup[3];		// The ThreadGroups this ThreadGroup contains
 
-	private static final class ChildrenGroupsLock {}
+	private static final class ChildrenGroupsLock { ChildrenGroupsLock() { super(); } }
 	// Locked when using the childrenGroups field
 	private Object childrenGroupsLock = new ChildrenGroupsLock();
-	private static final class ChildrenThreadsLock {}
+	private static final class ChildrenThreadsLock { ChildrenThreadsLock() { super(); } }
 	// Locked when using the childrenThreads field
 	private Object childrenThreadsLock = new ChildrenThreadsLock();
 
 	private boolean isDaemon;			// Whether this ThreadGroup is a daemon ThreadGroup or not	
 	private boolean isDestroyed;			// Whether this ThreadGroup has already been destroyed or not	
 	private int addedNotStartedThreads;	// Threads that have been added but not yet started
-	private long memorySpace;				// Memory space to associate all new threads with
 
 /**
  * Used by the JVM to create the "system" ThreadGroup. Construct
  * a ThreadGroup instance, and assign the name "system".
  */
+@SuppressWarnings("unused")
 private ThreadGroup() {
 	name = "system"; //$NON-NLS-1$
 }
@@ -252,7 +248,7 @@ public final void checkAccess() {
  * @throws		IllegalThreadStateException 	if the receiver or any of its subgroups has been destroyed already
  * @throws		SecurityException 				if <code>this.checkAccess()</code> fails with a SecurityException
  */
-public final void destroy(){
+public final void destroy() {
 	destroyImpl();
 	removeFromParent();
 }
@@ -544,7 +540,6 @@ public final void interrupt() {
  * @see			#setDaemon 
  * @see			#destroy
  */
- 
 public final boolean isDaemon() {
 	return isDaemon;
 }
@@ -556,7 +551,7 @@ public final boolean isDaemon() {
  *
  * @see			#destroy
  */
-public boolean isDestroyed(){
+public boolean isDestroyed() {
 	// never call this when synchronized on childrenThreadsGroup or deadlock will occur
 	synchronized(childrenThreadsLock) {
 		return isDestroyed;
@@ -709,7 +704,6 @@ public final void resume() {
  * @see			#isDaemon 
  * @see			#destroy
  */
- 
 public final void setDaemon(boolean isDaemon) {
 	checkAccess();
 	this.isDaemon = isDaemon;
@@ -768,7 +762,7 @@ private void setParent(ThreadGroup parent) {
  *
  * @deprecated Requires deprecated method Thread.stop().
  */
-@Deprecated(forRemoval=false, since="1.2")
+@Deprecated(forRemoval = false, since = "1.2")
 public final void stop() {
 	if (stopHelper())
 		Thread.currentThread().stop();
@@ -843,7 +837,7 @@ private final boolean suspendHelper() {
  *
  * @return		a printable representation for the receiver.
  */
- 
+@Override
 public String toString() {
 	return getClass().getName() + "[name=" + this.getName() + ",maxpri=" + this.getMaxPriority() + "]" ;  //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
 }
@@ -860,6 +854,7 @@ public String toString() {
  * @see			Thread#stop(Throwable)
  * @see			ThreadDeath
  */
+@Override
 public void uncaughtException(Thread t, Throwable e) {
 	Thread.UncaughtExceptionHandler handler;
 	if (parent != null) {
@@ -874,4 +869,3 @@ public void uncaughtException(Thread t, Throwable e) {
 	}
 }
 }
-
