@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2009, 2019 IBM Corp. and others
+ * Copyright (c) 2009, 2020 IBM Corp. and others
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -60,36 +60,19 @@ import java.lang.invoke.MethodTypeForm;
 public final class MethodType implements Serializable 
 {
 	static final Class<?>[] EMTPY_PARAMS = new Class<?>[0];
-	static final Set<Class<?>> WRAPPER_SET;
-	static {
-		Class<?>[] wrappers = {Byte.class, Character.class, Double.class, Float.class, Integer.class, Long.class, Short.class, Boolean.class };
-		WRAPPER_SET = Collections.unmodifiableSet(new HashSet<Class<?>>(Arrays.asList(wrappers)));
-	}
 
 	private MethodTypeForm form;
-	
-	static final Class<?>[] primitivesArray = new Class<?>[26];
-	static {
-		primitivesArray['B' - 'A'] = byte.class;
-		primitivesArray['C' - 'A'] = char.class;
-		primitivesArray['D' - 'A'] = double.class;
-		primitivesArray['F' - 'A'] = float.class;
-		primitivesArray['I' - 'A'] = int.class;
-		primitivesArray['J' - 'A'] = long.class;
-		primitivesArray['S' - 'A'] = short.class;
-		primitivesArray['Z' - 'A'] = boolean.class;
-	}
-	
+
 	private static Map<MethodType, WeakReference<MethodType>> internTable = Collections.synchronizedMap(new WeakHashMap<MethodType, WeakReference<MethodType>>());
 	// lock object to control insertion of new MTs
 	static final class InternTableAddLock { }
 	private static InternTableAddLock internTableAddLock = new InternTableAddLock();
 	
 	@VMCONSTANTPOOL_FIELD
-	final Class<?> returnType;
+	final Class<?> rtype;
 	
 	@VMCONSTANTPOOL_FIELD
-	final Class<?>[] arguments;
+	final Class<?>[] ptypes;
 
 	@VMCONSTANTPOOL_FIELD
 	int argSlots;	// Number of stack slots used by the described args in the MethodType
@@ -104,17 +87,32 @@ public final class MethodType implements Serializable
 	/* Indicate that there are no persistent/serializable fields in MethodType */
 	private static final ObjectStreamField[] serialPersistentFields = new ObjectStreamField[0];
 	private static final long serialVersionUID = 292L;
+	private DeserializedFieldsHolder deserializedFields;
+
+	/**
+	 * Holder for the deserialized arguments[] and returnType values
+	 * from #readObject() so they can be passed to #readResolve()
+	 */
+	private static class DeserializedFieldsHolder {
+		Class<?> returnType;
+		Class<?>[] arguments;
+
+		DeserializedFieldsHolder(Class<?> rtype, Class<?>[] args) {
+			returnType = (rtype == null) ? void.class : rtype;
+			arguments = (args == null) ? EMTPY_PARAMS : args;
+		}
+	}
 
 	/*
 	 * Private constructor as MethodTypes need to be interned.  
 	 */
 	private MethodType(Class<?> rtype, Class<?>[] ptypes, boolean copyArguments) {
-		returnType = rtype;
+		this.rtype = rtype;
 		if (copyArguments) {
-			arguments = new Class<?>[ptypes.length];
-			System.arraycopy(ptypes, 0, arguments, 0, ptypes.length);
+			this.ptypes = new Class<?>[ptypes.length];
+			System.arraycopy(ptypes, 0, this.ptypes, 0, ptypes.length);
 		} else {
-			arguments = ptypes;
+			this.ptypes = ptypes;
 		}
 	}
 		
@@ -180,9 +178,9 @@ public final class MethodType implements Serializable
 	 */
 	public MethodType changeParameterType(int position, Class<?> type) {
 		//TODO: Should check that position is valid.
-		Class<?>[] newParameters = arguments.clone();
+		Class<?>[] newParameters = ptypes.clone();
 		newParameters[position] = type;
-		return MethodType.methodType(returnType, newParameters, false); 
+		return MethodType.methodType(rtype, newParameters, false); 
 	 }
 	
 	/**
@@ -192,7 +190,7 @@ public final class MethodType implements Serializable
 	 * @return a new MethodType with a changed return type
 	 */
 	public MethodType changeReturnType(Class<?> type){
-		return methodType(type, arguments, false);
+		return methodType(type, ptypes, false);
 	}
    
 	/**
@@ -205,22 +203,22 @@ public final class MethodType implements Serializable
 	 * @throws IndexOutOfBoundsException if the startPosition or endPosition are not valid indexes or if the startPosition is greater than the endPosition
      */
     public MethodType dropParameterTypes(int startPosition, int endPosition) throws IndexOutOfBoundsException {
-		if ((startPosition >= 0) && (endPosition >= 0) && (startPosition <= endPosition) && (endPosition <= arguments.length)) {
+		if ((startPosition >= 0) && (endPosition >= 0) && (startPosition <= endPosition) && (endPosition <= ptypes.length)) {
     		int delta = endPosition - startPosition;
-    		Class<?>[] newParameters = new Class<?>[arguments.length - delta];
-    		System.arraycopy(arguments, 0, newParameters, 0, startPosition);
-    		System.arraycopy(arguments, endPosition, newParameters, startPosition, arguments.length - endPosition);
-    		return methodType(returnType, newParameters, false);
+    		Class<?>[] newParameters = new Class<?>[ptypes.length - delta];
+    		System.arraycopy(ptypes, 0, newParameters, 0, startPosition);
+    		System.arraycopy(ptypes, endPosition, newParameters, startPosition, ptypes.length - endPosition);
+    		return methodType(rtype, newParameters, false);
 		}
     	throw new IndexOutOfBoundsException("'" + this + "' startPosition=" + startPosition + " endPosition=" + endPosition);  //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
 	}
     
     /* Internal helper method that is equivalent to dropParameterTypes(0, 1) */
     MethodType dropFirstParameterType() throws IndexOutOfBoundsException {
-    	if (arguments.length == 0) {
+    	if (ptypes.length == 0) {
     		throw new IndexOutOfBoundsException();
     	}
-    	return methodType(returnType, Arrays.copyOfRange(arguments, 1, arguments.length), false);
+    	return methodType(rtype, Arrays.copyOfRange(ptypes, 1, ptypes.length), false);
 	}
     
 	  
@@ -240,10 +238,10 @@ public final class MethodType implements Serializable
 		}
 
 		MethodType that = (MethodType) x;
-		if ((this.arguments.length != that.arguments.length) || (this.returnType != that.returnType)) {
+		if ((this.ptypes.length != that.ptypes.length) || (this.rtype != that.rtype)) {
 			return false;
 		}
-		return Arrays.equals(this.arguments, that.arguments);
+		return Arrays.equals(this.ptypes, that.ptypes);
 	}
 	
 	/**
@@ -253,13 +251,13 @@ public final class MethodType implements Serializable
 	 * @return a new MethodType with all non-primitive or void return type and parameters changed to Object.
 	 */
 	public MethodType erase() {
-		Class<?>[] newParameters = arguments.clone();
+		Class<?>[] newParameters = ptypes.clone();
 		for (int i= 0; i < newParameters.length; i++){
 			if (!newParameters[i].isPrimitive()) {
 				newParameters[i] = Object.class;
 			}
 		}
-		return methodType(returnType.isPrimitive() ? returnType : Object.class, newParameters, false);
+		return methodType(rtype.isPrimitive() ? rtype : Object.class, newParameters, false);
 	}
 	
 	
@@ -337,32 +335,6 @@ public final class MethodType implements Serializable
 		
 		return methodType(returnType, types);
 	}
-	
-	/**
-	 * This helper calls MethodType.fromMethodDescriptorString(...) or 
-	 * MethodType.fromMethodDescriptorStringAppendArg(...) but throws 
-	 * NoClassDefFoundError instead of TypeNotPresentException during 
-	 * the VM resolve stage.
-	 *
-	 * @param methodDescriptor - the method descriptor string
-	 * @param loader - the ClassLoader to be used
-	 * @param appendArgumentType - an extra argument type
-	 *
-	 * @return a MethodType object representing the method descriptor string
-	 *
-	 * @throws IllegalArgumentException - if the string is not well-formed
-	 * @throws NoClassDefFoundError - if a named type cannot be found
-	 */
-	static final MethodType vmResolveFromMethodDescriptorString(String methodDescriptor, ClassLoader loader, Class<?> appendArgumentType) throws Throwable {
-		try {
-			if (null == appendArgumentType) {
-				return MethodType.fromMethodDescriptorString(methodDescriptor, loader);
-			}
-			return MethodType.fromMethodDescriptorStringAppendArg(methodDescriptor, loader, appendArgumentType);
-		} catch (TypeNotPresentException e) {
-			throw throwNoClassDefFoundError(e);
-		}
-	}
 
 	/**
 	 * Helper method to throw NoClassDefFoundError if the cause of TypeNotPresentException 
@@ -384,72 +356,6 @@ public final class MethodType implements Serializable
 		}
 		throw e;
 	}
-	
-	/*
-	 * Convert the string from bytecode format to the format needed for ClassLoader#loadClass().
-	 * Change all '/' to '.'.
-	 * Remove the 'L' & ';' from objects, unless they are array classes.
-	 */
-	private static final Class<?> nonPrimitiveClassFromString(String name, ClassLoader classLoader) {
-		try {
-			name = name.replace('/', '.');
-			if (name.indexOf('L') == 0) {
-				// Remove the 'L' and ';'
-				name = name.substring(1, name.length() - 1);
-			}
-			return Class.forName(name, false, classLoader);
-		} catch(ClassNotFoundException e){
-			throw new TypeNotPresentException(name, e);
-		}
-	}
-
-	/*
-	 * Parses the first parameter in a descriptor string into a Class object.
-	 * The Class object is append to given ArrayList and the current string index is returned.
-	 */
-	static final int parseIntoClass(char[] signature, int index, ArrayList<Class<?>> args, ClassLoader classLoader, String descriptor) {
-		char current = signature[index];
-		Class<?> c;			
-		
-		if ((current == 'L') || (current == '[')) {
-			int start = index;
-			while(signature[index] == '[') {
-				index++;
-			}
-			String name;
-			if (signature[index] != 'L') {
-				name = descriptor.substring(start, index + 1);
-			} else { 
-				int end = descriptor.indexOf(';', index);
-				if (end == -1) {
-					// K05d6 = malformed method descriptor: {0}
-					throw new IllegalArgumentException(Msg.getString("K05d6", descriptor)); //$NON-NLS-1$
-				}
-				name = descriptor.substring(start, end + 1);
-				index = end;
-			}
-			c = nonPrimitiveClassFromString(name, classLoader);
-		} else {
-			try {
-				c = primitivesArray[current - 'A'];
-			} catch(ArrayIndexOutOfBoundsException e) {
-				c = null;
-			}
-			if (c == null) {
-				if (current == 'V') {
-					// lazily add 'V' to work around load ordering issues
-					primitivesArray['V' - 'A'] = void.class;
-					c = void.class;
-				} else {
-					// K05d7 = not a primitive: {0}
-					throw new IllegalArgumentException(Msg.getString("K05d7", current)); //$NON-NLS-1$
-				}
-			}
-		}
-		args.add(c);
-
-		return index;
-	}
 
 	/*
 	 * Parse the MethodDescriptor string into a list of Class objects.  The last class in the list
@@ -487,7 +393,7 @@ public final class MethodType implements Serializable
 				continue;
 			}
 
-			index = parseIntoClass(signature, index, args, classLoader, methodDescriptor);
+			index = MethodTypeHelper.parseIntoClass(signature, index, args, classLoader, methodDescriptor);
 			index++;
 		}
 		return args;
@@ -500,7 +406,7 @@ public final class MethodType implements Serializable
 	 * @return a new MethodType with both return and parameter types changed to Object
 	 */
 	public MethodType generic() {
-		return genericMethodType(arguments.length);
+		return genericMethodType(ptypes.length);
 	}
 	
 	
@@ -513,8 +419,8 @@ public final class MethodType implements Serializable
 	 */
 	public int hashCode(){
 		if (hashcode == 0) {
-			int hash = 31 + returnType.hashCode();
-			for (Class<?> c : arguments) {
+			int hash = 31 + rtype.hashCode();
+			for (Class<?> c : ptypes) {
 				hash = 31 * hash + c.hashCode();
 			}
 			hashcode = hash;
@@ -530,10 +436,10 @@ public final class MethodType implements Serializable
 	 * @return whether the MethodType contains any primitive types
 	 */
 	public boolean hasPrimitives(){
-		if (returnType.isPrimitive()){
+		if (rtype.isPrimitive()){
 			return true;
 		}
-		for(Class<?> c : arguments){
+		for(Class<?> c : ptypes){
 			if (c.isPrimitive()) {
 				return true;
 			}
@@ -551,11 +457,11 @@ public final class MethodType implements Serializable
 	 * @return whether the MethodType contains any wrapper types
 	 */
 	public boolean hasWrappers() {
-		if (WRAPPER_SET.contains(returnType) || (returnType == Void.class)){
+		if (MethodTypeHelper.WRAPPER_SET.contains(rtype) || (rtype == Void.class)){
 			return true;
 		}
-		for(Class<?> c : arguments){
-			if (WRAPPER_SET.contains(c)){
+		for(Class<?> c : ptypes){
+			if (MethodTypeHelper.WRAPPER_SET.contains(c)){
 				return true;
 			}
 		}
@@ -572,7 +478,7 @@ public final class MethodType implements Serializable
 	 * @throws IndexOutOfBoundsException if position is less than 0 or greater than than the number of arguments
 	 */
 	public MethodType insertParameterTypes(int position, Class<?>... types) throws IndexOutOfBoundsException {
-		if (position < 0 || position > arguments.length) {
+		if (position < 0 || position > ptypes.length) {
 			throw new IndexOutOfBoundsException();
 		}
 		int typesLength = types.length;
@@ -581,14 +487,14 @@ public final class MethodType implements Serializable
 			return this;
 		}
 		
-		Class<?>[] params = new Class<?>[arguments.length + typesLength];
+		Class<?>[] params = new Class<?>[ptypes.length + typesLength];
 		// Copy from 0 -> position out of the existing MethodType arguments
-		System.arraycopy(arguments, 0, params, 0, position);
+		System.arraycopy(ptypes, 0, params, 0, position);
 		// Copy from position -> types.length out of the types[] array
 		System.arraycopy(types, 0, params, position, typesLength);
 		// Copy from position + types.length -> params.length out of the existing MT arguments
-		System.arraycopy(arguments, position, params, position + typesLength, arguments.length - position);
-		return methodType(returnType, params, false);
+		System.arraycopy(ptypes, position, params, position + typesLength, ptypes.length - position);
+		return methodType(rtype, params, false);
 	}
 	
 	/**
@@ -674,9 +580,9 @@ public final class MethodType implements Serializable
 				return type;
 			}
 			
-			int stackSlots = arguments.length;
+			int stackSlots = ptypes.length;
 			
-			for(Class<?> c : arguments) {
+			for(Class<?> c : ptypes) {
 				c.getClass();	// Implicit nullcheck
 				if ((c == double.class) || (c == long.class)) {
 					stackSlots++;
@@ -692,7 +598,7 @@ public final class MethodType implements Serializable
 			argSlots = stackSlots;
 
 			/* initialize expensive state */
-			stackDescriptionBits = stackDescriptionBits(arguments, argSlots);
+			stackDescriptionBits = stackDescriptionBits(ptypes, argSlots);
 			methodDescriptor = createMethodDescriptorString();
 
 			MethodType tenured = makeTenured(this);
@@ -751,7 +657,7 @@ public final class MethodType implements Serializable
 	 * @return a MethodType object made by changing the return type of the passed in MethodType
 	 */
 	public static MethodType methodType(Class<?> returnType, MethodType methodType) {
-		return methodType(returnType, methodType.arguments, false);
+		return methodType(returnType, methodType.ptypes, false);
 	}
 
 	/**
@@ -802,7 +708,7 @@ public final class MethodType implements Serializable
 	 * @return the parameter types as an array
 	 */
 	public Class<?>[] parameterArray() {
-		return arguments.clone();
+		return ptypes.clone();
 	}
 
 	/**
@@ -811,7 +717,16 @@ public final class MethodType implements Serializable
 	 * @return the number of parameters
 	 */
 	public int parameterCount() {
-		return arguments.length;
+		return ptypes.length;
+	}
+
+	/**
+	 * Method to return array of arguments
+	 *
+	 * @return array of arguments
+	 */
+	Class<?>[] ptypes() {
+		return ptypes;
 	}
 
 	/**
@@ -822,7 +737,7 @@ public final class MethodType implements Serializable
 	 * @return the parameter types as a List
 	 */
 	public List<Class<?>> parameterList() {
-		List<Class<?>> list = Arrays.asList(arguments.clone()); 
+		List<Class<?>> list = Arrays.asList(ptypes.clone()); 
 		return Collections.unmodifiableList(list);
 	}     
 
@@ -834,14 +749,14 @@ public final class MethodType implements Serializable
 	 * @throws IndexOutOfBoundsException if position is less than 0 or an invalid argument index.
 	 */
 	public Class<?> parameterType(int position) throws IndexOutOfBoundsException {
-		return arguments[position];
+		return ptypes[position];
 	}
 	
 	/**
 	 * @return the type of the return
 	 */
 	public Class<?> returnType() {
-		return returnType;
+		return rtype;
 	}
 	
 	/**
@@ -853,8 +768,8 @@ public final class MethodType implements Serializable
 	public
 	Class<?> lastParameterType() {
 		Class<?> result = void.class;
-		if (arguments.length > 0) {
-			result = arguments[arguments.length - 1];
+		if (ptypes.length > 0) {
+			result = ptypes[ptypes.length - 1];
 		}
 		return result;
 	}
@@ -871,11 +786,11 @@ public final class MethodType implements Serializable
 	
 	private String createMethodDescriptorString(){
 		StringBuilder sb= new StringBuilder("("); //$NON-NLS-1$
-		for (Class<?> c : arguments){
-			sb.append(getBytecodeStringName(c));
+		for (Class<?> c : ptypes){
+			sb.append(MethodTypeHelper.getBytecodeStringName(c));
 		}
 		sb.append(")"); //$NON-NLS-1$
-		sb.append(getBytecodeStringName(returnType));
+		sb.append(MethodTypeHelper.getBytecodeStringName(rtype));
 		return sb.toString();
 	}
 	
@@ -889,14 +804,14 @@ public final class MethodType implements Serializable
 	@Override
 	public String toString(){
 		StringBuilder sb = new StringBuilder("("); //$NON-NLS-1$
-		for (int i = 0; i < arguments.length; i++){
-			sb.append(arguments[i].getSimpleName());
-			if (i < (arguments.length - 1)) {
+		for (int i = 0; i < ptypes.length; i++){
+			sb.append(ptypes[i].getSimpleName());
+			if (i < (ptypes.length - 1)) {
 				sb.append(","); //$NON-NLS-1$
 			}
 		}
 		sb.append(")"); //$NON-NLS-1$
-		sb.append(returnType.getSimpleName());
+		sb.append(rtype.getSimpleName());
 		return sb.toString();
 	}
 
@@ -909,12 +824,12 @@ public final class MethodType implements Serializable
 	 * @see #wrap()
 	 */
 	public MethodType unwrap(){
-		Class<?>[] args = arguments.clone();
+		Class<?>[] args = ptypes.clone();
 		for(int i = 0; i < args.length; i++){
-			args[i] = unwrapPrimitive(args[i]);
+			args[i] = MethodTypeHelper.unwrapPrimitive(args[i]);
 		}
-		Class<?> unwrappedReturnType = unwrapPrimitive(returnType);
-		if (unwrapPrimitive(returnType) == Void.class) {
+		Class<?> unwrappedReturnType = MethodTypeHelper.unwrapPrimitive(rtype);
+		if (MethodTypeHelper.unwrapPrimitive(rtype) == Void.class) {
 			unwrappedReturnType = void.class;
 		}
 		return methodType(unwrappedReturnType, args, false);
@@ -929,11 +844,11 @@ public final class MethodType implements Serializable
 	 * @see #unwrap()
 	 */
 	public MethodType wrap(){
-		Class<?>[] args = arguments.clone();
+		Class<?>[] args = ptypes.clone();
 		for(int i = 0; i < args.length; i++){
-			args[i] = wrapPrimitive(args[i]);
+			args[i] = MethodTypeHelper.wrapPrimitive(args[i]);
 		}
-		return methodType(wrapPrimitive(returnType), args, false);
+		return methodType(MethodTypeHelper.wrapPrimitive(rtype), args, false);
 	}
 	
 	/**
@@ -963,112 +878,11 @@ public final class MethodType implements Serializable
 		if (classes == null) {
 			throw new NullPointerException();
 		}
-		ArrayList<Class<?>> combinedParameters = new ArrayList<Class<?>>(Arrays.asList(arguments));
+		ArrayList<Class<?>> combinedParameters = new ArrayList<Class<?>>(Arrays.asList(ptypes));
 		combinedParameters.addAll(classes);
-		return methodType(returnType, combinedParameters);
+		return methodType(rtype, combinedParameters);
 	}
-	
-	/**
-	 * Returns the appropriate wrapper class or the original class
-	 * @param primitiveClass The class to convert to a wrapper class
-	 * @return The wrapper class or the original class if no wrapper is available
-	 */
-	static Class<?> wrapPrimitive(Class<?> primitveClass) {
-		if (primitveClass.isPrimitive()) {
-			if (int.class == primitveClass){ //I
-				return Integer.class;
-			}
-			if (long.class == primitveClass){ //J
-				return Long.class;
-			}
-			if (byte.class == primitveClass){	//B
-				return Byte.class;
-			}
-			if (char.class == primitveClass){ //C
-				return Character.class;
-			}
-			if (double.class == primitveClass){ //D
-				return Double.class;
-			}
-			if (float.class == primitveClass){ //F
-				return Float.class;
-			}
-			if (boolean.class == primitveClass){ //Z
-				return Boolean.class;
-			}
-			if (void.class == primitveClass) {
-				return Void.class;
-			}
-			if (short.class == primitveClass){ //S
-				return Short.class;
-			}
-		}
-		return primitveClass;
-	}
-	
-	
-	/**
-	 * Takes a class and returns the corresponding primitive if one exists.
-	 * @param wrapperClass The class to check for primitive classes.
-	 * @return The corresponding primitive class or the input class.
-	 */
-	static Class<?> unwrapPrimitive(Class<?> wrapperClass){
-		if (Integer.class == wrapperClass) { //I
-			return int.class;
-		}
-		if (Long.class == wrapperClass) { //J
-			return long.class;
-		}
-		if (Byte.class == wrapperClass) {	//B
-			return byte.class;
-		}
-		if (Character.class == wrapperClass) { //C
-			return char.class;
-		}
-		if (Double.class == wrapperClass) { //D
-			return double.class;
-		}
-		if (Float.class == wrapperClass) { //F
-			return float.class;
-		}
-		if (Short.class == wrapperClass) { //S
-			return short.class;
-		}
-		if (Boolean.class == wrapperClass) { //Z
-			return boolean.class;
-		}
-		return wrapperClass;
-	}
-	
-	static String getBytecodeStringName(Class<?> c){
-		if (c.isPrimitive()) {
-			if (c == int.class) {
-				return "I"; //$NON-NLS-1$
-			} else if (c == long.class) {
-				return "J"; //$NON-NLS-1$
-			} else if (c == byte.class) {
-				return "B"; //$NON-NLS-1$
-			} else if (c == boolean.class) {
-				return "Z"; //$NON-NLS-1$
-			} else if (c == void.class) {
-				return "V"; //$NON-NLS-1$
-			} else if (c == char.class) {
-				return "C"; //$NON-NLS-1$
-			} else if (c == double.class) {
-				return "D"; //$NON-NLS-1$
-			} else if (c == float.class) {
-				return "F"; //$NON-NLS-1$
-			} else if (c == short.class) {
-				return "S"; //$NON-NLS-1$
-			}
-		}
-		String name = c.getName().replace('.', '/');
-		if (c.isArray()) {
-			return name;
-		}
-		return "L"+ name + ";"; //$NON-NLS-1$ //$NON-NLS-2$
-	}
-	
+
 	/*
 	 * Implements JSR 292 serialization spec.
 	 * Write the MethodType without writing the field names.
@@ -1090,8 +904,8 @@ public final class MethodType implements Serializable
 	 */
 	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
 		try {
-			final Field fReturnType = getClass().getDeclaredField("returnType"); //$NON-NLS-1$
-			final Field fArguments = getClass().getDeclaredField("arguments"); //$NON-NLS-1$
+			final Field fReturnType = getClass().getDeclaredField("rtype"); //$NON-NLS-1$
+			final Field fArguments = getClass().getDeclaredField("ptypes"); //$NON-NLS-1$
 			AccessController.doPrivileged(new PrivilegedAction<Object>() {
 				public Object run() {
 						fReturnType.setAccessible(true);
@@ -1100,14 +914,13 @@ public final class MethodType implements Serializable
 					}
 				}
 			);
-			try {
-				fReturnType.set(this, in.readObject());
-				fArguments.set(this, in.readObject());
-			} catch(Exception e) {
-				fReturnType.set(this, void.class);
-				fArguments.set(this, EMTPY_PARAMS);
-				throw e;
-			}
+			fReturnType.set(this, void.class);
+			fArguments.set(this, EMTPY_PARAMS);
+			methodDescriptor = "()V";
+			stackDescriptionBits = stackDescriptionBits(EMTPY_PARAMS, 0);
+			Class<?> serialReturnType = (Class<?>) in.readObject();
+			Class<?>[] serialArguments = (Class<?>[]) in.readObject();
+			deserializedFields = new DeserializedFieldsHolder(serialReturnType, serialArguments);
 		} catch (IllegalAccessException e) {
 		} catch (NoSuchFieldException e) {
 		}
@@ -1116,7 +929,14 @@ public final class MethodType implements Serializable
 	/* Hook to ensure that all objects go through the factory */
 	@SuppressWarnings("unused")
 	private Object readResolve() throws ObjectStreamException {
-		return MethodType.methodType(returnType, arguments);
+		Class<?> ret = void.class;
+		Class<?>[] args = EMTPY_PARAMS;
+		DeserializedFieldsHolder localArgs = deserializedFields;
+		if (localArgs != null) {
+			ret = deserializedFields.returnType;
+			args = deserializedFields.arguments;
+		}
+		return MethodType.methodType(ret, args);
 	}
 	
 	/*
@@ -1134,7 +954,6 @@ public final class MethodType implements Serializable
 		}
 		return invoker;
 	}
-	
 
 	MethodType basicType() {
 		throw OpenJDKCompileStub.OpenJDKCompileStubThrowError();
@@ -1153,7 +972,7 @@ public final class MethodType implements Serializable
 	}
 	
 	Class<?> rtype() {
-		return returnType;
+		return rtype;
 	}
 
 	MethodType asCollectorType(Class<?> clz, int num1, int num2) {
@@ -1169,4 +988,3 @@ public final class MethodType implements Serializable
 	}
 
 }
-
