@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2019, 2019 IBM Corp. and others
+ * Copyright (c) 2019, 2020 IBM Corp. and others
  *
  * This program and the accompanying materials are made available under
  * the terms of the Eclipse Public License 2.0 which accompanies this
@@ -20,8 +20,11 @@
  * SPDX-License-Identifier: EPL-2.0 OR Apache-2.0 OR GPL-2.0 WITH Classpath-exception-2.0 OR LicenseRef-GPL-2.0 WITH Assembly-exception
  *******************************************************************************/
 
+/**
+ * Defines the jcmd tool, used for obtaining information about a running JVM, or requesting a
+ * running JVM to do operations such as creating diagnotic files or performing a GC.
+ */
 module jdk.jcmd {
-  requires java.base;
   requires java.management;
   requires jdk.attach;
 }
